@@ -1,16 +1,18 @@
-/* packet.h
+/** @file
  * Definitions for packet disassembly structures and routines
  *
  * Wireshark - Network traffic analyzer
  * By Gerald Combs <gerald@wireshark.org>
  * Copyright 1998 Gerald Combs
  *
- * SPDX-License-Identifier: GPL-2.0+
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef __PACKET_H__
 #define __PACKET_H__
+#include <wireshark.h>
 
+#include <wiretap/wtap_opttypes.h>
 #include "proto.h"
 #include "tvbuff.h"
 #include "epan.h"
@@ -21,7 +23,6 @@
 #include "guid-utils.h"
 #include "tfs.h"
 #include "unit_strings.h"
-#include "ws_symbol_export.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -46,6 +47,12 @@ struct epan_range;
 	((guint)(offset) + (guint)(len) > (guint)(offset) && \
 	 (guint)(offset) + (guint)(len) <= (guint)(captured_len))
 
+/* 0 is case insenstive for backwards compatibility with tables that
+ * used FALSE or BASE_NONE for case sensitive, which was the default.
+ */
+#define STRING_CASE_SENSITIVE 0
+#define STRING_CASE_INSENSITIVE 1
+
 extern void packet_init(void);
 extern void packet_cache_proto_handles(void);
 extern void packet_cleanup(void);
@@ -165,9 +172,9 @@ WS_DLL_PUBLIC void dissector_all_tables_foreach_table (DATFunc_table func,
 
 /* a protocol uses the function to register a sub-dissector table
  *
- * 'param' is the display base for integer tables, and TRUE/FALSE for
- * string tables (true indicating case-insensitive, false indicating
- * case-sensitive)
+ * 'param' is the display base for integer tables, STRING_CASE_SENSITIVE
+ * or STRING_CASE_INSENSITIVE for string tables, and ignored for other
+ * table types.
  */
 WS_DLL_PUBLIC dissector_table_t register_dissector_table(const char *name,
     const char *ui_name, const int proto, const ftenum_t type, const int param);
@@ -177,7 +184,16 @@ WS_DLL_PUBLIC dissector_table_t register_dissector_table(const char *name,
  * to store subdissectors.
  */
 WS_DLL_PUBLIC dissector_table_t register_custom_dissector_table(const char *name,
-    const char *ui_name, const int proto, GHashFunc hash_func, GEqualFunc key_equal_func);
+    const char *ui_name, const int proto, GHashFunc hash_func, GEqualFunc key_equal_func,
+    GDestroyNotify key_destroy_func);
+
+/** Register a dissector table alias.
+ * This is for dissectors whose original name has changed, e.g. SSL to TLS.
+ * @param dissector_table dissector table returned by register_dissector_table.
+ * @param alias_name alias for the dissector table name.
+ */
+WS_DLL_PUBLIC void register_dissector_table_alias(dissector_table_t dissector_table,
+    const char *alias_name);
 
 /** Deregister the dissector table by table name. */
 void deregister_dissector_table(const char *name);
@@ -236,6 +252,13 @@ WS_DLL_PUBLIC void dissector_change_uint(const char *abbrev, const guint32 patte
 /* Reset an entry in a uint dissector table to its initial value. */
 WS_DLL_PUBLIC void dissector_reset_uint(const char *name, const guint32 pattern);
 
+/* Return TRUE if an entry in a uint dissector table is found and has been
+ * changed (i.e. dissector_change_uint() has been called, such as from
+ * Decode As, prefs registered via dissector_add_uint_[range_]with_preference),
+ * etc.), otherwise return FALSE.
+ */
+WS_DLL_PUBLIC gboolean dissector_is_uint_changed(dissector_table_t const sub_dissectors, const guint32 uint_val);
+
 /* Look for a given value in a given uint dissector table and, if found,
    call the dissector with the arguments supplied, and return the number
    of bytes consumed, otherwise return 0. */
@@ -285,6 +308,12 @@ WS_DLL_PUBLIC void dissector_change_string(const char *name, const gchar *patter
 /* Reset an entry in a string sub-dissector table to its initial value. */
 WS_DLL_PUBLIC void dissector_reset_string(const char *name, const gchar *pattern);
 
+/* Return TRUE if an entry in a string dissector table is found and has been
+ * changed (i.e. dissector_change_string() has been called, such as from
+ * Decode As), otherwise return FALSE.
+ */
+WS_DLL_PUBLIC gboolean dissector_is_string_changed(dissector_table_t const subdissectors, const gchar *string);
+
 /* Look for a given string in a given dissector table and, if found, call
    the dissector with the arguments supplied, and return the number of
    bytes consumed, otherwise return 0. */
@@ -383,6 +412,12 @@ WS_DLL_PUBLIC void dissector_change_payload(const char *abbrev, dissector_handle
 /* Reset payload (FT_NONE) dissector table to its initial value. */
 WS_DLL_PUBLIC void dissector_reset_payload(const char *name);
 
+/* Given a payload dissector table (type FT_NONE), return the handle of
+   the dissector that is currently active, i.e. that was selected via
+   Decode As. */
+WS_DLL_PUBLIC dissector_handle_t dissector_get_payload_handle(
+        dissector_table_t const dissector_table);
+
 /* Add a handle to the list of handles that *could* be used with this
    table.  That list is used by the "Decode As"/"-d" code in the UI. */
 WS_DLL_PUBLIC void dissector_add_for_decode_as(const char *name,
@@ -396,9 +431,10 @@ WS_DLL_PUBLIC void dissector_add_for_decode_as_with_preference(const char *name,
  */
 WS_DLL_PUBLIC GSList *dissector_table_get_dissector_handles(dissector_table_t dissector_table);
 
-/** Get a handle to dissector out of a dissector table
+/** Get a handle to dissector out of a dissector table given the description
+ * of what the dissector dissects.
  */
-WS_DLL_PUBLIC dissector_handle_t dissector_table_get_dissector_handle(dissector_table_t dissector_table, gchar* short_name);
+WS_DLL_PUBLIC dissector_handle_t dissector_table_get_dissector_handle(dissector_table_t dissector_table, const gchar* description);
 
 /** Get a dissector table's type
  */
@@ -408,6 +444,10 @@ WS_DLL_PUBLIC ftenum_t dissector_table_get_type(dissector_table_t dissector_tabl
  */
 WS_DLL_PUBLIC void dissector_table_allow_decode_as(dissector_table_t dissector_table);
 
+/** Returns TRUE if dissector table allows "Decode As"
+ */
+WS_DLL_PUBLIC gboolean dissector_table_supports_decode_as(dissector_table_t dissector_table);
+
 /* List of "heuristic" dissectors (which get handed a packet, look at it,
    and either recognize it as being for their protocol, dissect it, and
    return TRUE, or don't recognize it and return FALSE) to be called
@@ -425,12 +465,14 @@ typedef struct heur_dtbl_entry {
 	const gchar *display_name;     /* the string used to present heuristic to user */
 	gchar *short_name;     /* string used for "internal" use to uniquely identify heuristic */
 	gboolean enabled;
+	bool enabled_by_default;
 } heur_dtbl_entry_t;
 
 /** A protocol uses this function to register a heuristic sub-dissector list.
  *  Call this in the parent dissectors proto_register function.
  *
  * @param name the name of this protocol
+ * @param proto the value obtained when registering the protocol
  */
 WS_DLL_PUBLIC heur_dissector_list_t register_heur_dissector_list(const char *name, const int proto);
 
@@ -462,7 +504,7 @@ WS_DLL_PUBLIC void heur_dissector_table_foreach(const char *table_name,
 WS_DLL_PUBLIC void dissector_all_heur_tables_foreach_table (DATFunc_heur_table func,
     gpointer user_data, GCompareFunc compare_key_func);
 
-/* true if a heur_dissector list of that anme exists to be registered into */
+/* true if a heur_dissector list of that name exists to be registered into */
 WS_DLL_PUBLIC gboolean has_heur_dissector_list(const gchar *name);
 
 /** Try all the dissectors in a given heuristic dissector list. This is done,
@@ -497,15 +539,15 @@ WS_DLL_PUBLIC heur_dtbl_entry_t* find_heur_dissector_by_unique_short_name(const
 /** Add a sub-dissector to a heuristic dissector list.
  *  Call this in the proto_handoff function of the sub-dissector.
  *
- * @param name the name of the "parent" protocol, e.g. "tcp"
+ * @param name the name of the heuristic dissector table into which to register the dissector, e.g. "tcp"
  * @param dissector the sub-dissector to be registered
  * @param display_name the string used to present heuristic to user, e.g. "HTTP over TCP"
- * @param short_name the string used for "internal" use to identify heuristic, e.g. "http_tcp"
+ * @param internal_name the string used for "internal" use to identify heuristic, e.g. "http_tcp"
  * @param proto the protocol id of the sub-dissector
  * @param enable initially enabled or not
  */
 WS_DLL_PUBLIC void heur_dissector_add(const char *name, heur_dissector_t dissector,
-    const char *display_name, const char *short_name, const int proto, heuristic_enable_e enable);
+    const char *display_name, const char *internal_name, const int proto, heuristic_enable_e enable);
 
 /** Remove a sub-dissector from a heuristic dissector list.
  *  Call this in the prefs_reinit function of the sub-dissector.
@@ -519,6 +561,9 @@ WS_DLL_PUBLIC void heur_dissector_delete(const char *name, heur_dissector_t diss
 /** Register a new dissector. */
 WS_DLL_PUBLIC dissector_handle_t register_dissector(const char *name, dissector_t dissector, const int proto);
 
+/** Register a new dissector with a description. */
+WS_DLL_PUBLIC dissector_handle_t register_dissector_with_description(const char *name, const char *description, dissector_t dissector, const int proto);
+
 /** Register a new dissector with a callback pointer. */
 WS_DLL_PUBLIC dissector_handle_t register_dissector_with_data(const char *name, dissector_cb_t dissector, const int proto, void *cb_data);
 
@@ -526,11 +571,18 @@ WS_DLL_PUBLIC dissector_handle_t register_dissector_with_data(const char *name,
 void deregister_dissector(const char *name);
 
 /** Get the long name of the protocol for a dissector handle. */
-extern const char *dissector_handle_get_long_name(const dissector_handle_t handle);
+WS_DLL_PUBLIC const char *dissector_handle_get_protocol_long_name(const dissector_handle_t handle);
 
 /** Get the short name of the protocol for a dissector handle. */
+WS_DLL_PUBLIC const char *dissector_handle_get_protocol_short_name(const dissector_handle_t handle);
+
+/* For backwards source and binary compatibility */
+G_DEPRECATED_FOR(dissector_handle_get_protocol_short_name)
 WS_DLL_PUBLIC const char *dissector_handle_get_short_name(const dissector_handle_t handle);
 
+/** Get the description for what the dissector for a dissector handle dissects. */
+WS_DLL_PUBLIC const char *dissector_handle_get_description(const dissector_handle_t handle);
+
 /** Get the index of the protocol for a dissector handle. */
 WS_DLL_PUBLIC int dissector_handle_get_protocol_index(const dissector_handle_t handle);
 
@@ -540,7 +592,7 @@ WS_DLL_PUBLIC GList* get_dissector_names(void);
 /** Find a dissector by name. */
 WS_DLL_PUBLIC dissector_handle_t find_dissector(const char *name);
 
-/** Find a dissector by name and add parent protocol as a depedency*/
+/** Find a dissector by name and add parent protocol as a dependency. */
 WS_DLL_PUBLIC dissector_handle_t find_dissector_add_dependency(const char *name, const int parent_proto);
 
 /** Get a dissector name from handle. */
@@ -551,6 +603,13 @@ WS_DLL_PUBLIC dissector_handle_t create_dissector_handle(dissector_t dissector,
     const int proto);
 WS_DLL_PUBLIC dissector_handle_t create_dissector_handle_with_name(dissector_t dissector,
     const int proto, const char* name);
+WS_DLL_PUBLIC dissector_handle_t create_dissector_handle_with_name_and_description(dissector_t dissector,
+    const int proto, const char* name, const char* description);
+WS_DLL_PUBLIC dissector_handle_t create_dissector_handle_with_data(dissector_cb_t dissector,
+    const int proto, void* cb_data);
+
+/* Dump all registered dissectors to the standard output */
+WS_DLL_PUBLIC void dissector_dump_dissectors(void);
 
 /** Call a dissector through a handle and if no dissector was found
  * pass it over to the "data" dissector instead.
@@ -618,7 +677,7 @@ WS_DLL_PUBLIC gboolean register_depend_dissector(const char* parent, const char*
 /** Unregister a protocol dependency
  * This is done automatically when removing from a dissector or
  * heuristic table.  This is for "manual" deregistration for things
- * like Lua
+ * like Lua.
  *
  *   @param parent "Parent" protocol short name
  *   @param dependent "Dependent" protocol short name
@@ -655,16 +714,17 @@ WS_DLL_PUBLIC void set_actual_length(tvbuff_t *tvb, const guint specified_len);
 WS_DLL_PUBLIC void register_init_routine(void (*func)(void));
 
 /**
- * Allows protocols to register "cleanup" routines which are called
+ * Allows protocols to register "cleanup" routines, which are called
  * after closing a capture file (or when preferences are changed, in
  * that case these routines are called before the init routines are
  * executed). It can be used to release resources that are allocated in
- * register_init_routine.
+ * an "init" routine.
  */
 WS_DLL_PUBLIC void register_cleanup_routine(void (*func)(void));
 
 /*
- * Register a shutdown routine to call once just before program exit
+ * Allows protocols to register "shutdown" routines, which are called
+ * once, just before program exit
  */
 WS_DLL_PUBLIC void register_shutdown_routine(void (*func)(void));
 
@@ -689,7 +749,7 @@ WS_DLL_PUBLIC void postseq_cleanup_all_protocols(void);
  * subsystems, liked dfilters, have finished initializing. This is
  * useful for dissector registration routines which need to compile
  * display filters. dfilters can't initialize itself until all protocols
- * have registereed themselvs. */
+ * have registered themselves. */
 WS_DLL_PUBLIC void
 register_final_registration_routine(void (*func)(void));
 
@@ -721,17 +781,17 @@ extern void free_data_sources(packet_info *pinfo);
 
 /* Mark another frame as depended upon by the current frame.
  *
- * This information is used to ensure that the dependend-upon frame is saved
+ * This information is used to ensure that the depended-upon frame is saved
  * if the user does a File->Save-As of only the Displayed packets and the
  * current frame passed the display filter.
  */
-WS_DLL_PUBLIC void mark_frame_as_depended_upon(packet_info *pinfo, guint32 frame_num);
+WS_DLL_PUBLIC void mark_frame_as_depended_upon(frame_data *fd, guint32 frame_num);
 
 /* Structure passed to the frame dissector */
 typedef struct frame_data_s
 {
     int file_type_subtype;
-    const gchar  *pkt_comment; /**< NULL if not available */
+    wtap_block_t pkt_block;         /**< NULL if not available */
     struct epan_dissect *color_edt; /** Used strictly for "coloring rules" */
 
 } frame_data_t;
@@ -739,7 +799,7 @@ typedef struct frame_data_s
 /* Structure passed to the file dissector */
 typedef struct file_data_s
 {
-    const gchar  *pkt_comment; /**< NULL if not available */
+    wtap_block_t pkt_block;         /**< NULL if not available */
     struct epan_dissect *color_edt; /** Used strictly for "coloring rules" */
 
 } file_data_t;
@@ -748,23 +808,20 @@ typedef struct file_data_s
  * Dissectors should never modify the record data.
  */
 extern void dissect_record(struct epan_dissect *edt, int file_type_subtype,
-    struct wtap_pkthdr *phdr, tvbuff_t *tvb,
-    frame_data *fd, column_info *cinfo);
+    wtap_rec *rec, tvbuff_t *tvb, frame_data *fd, column_info *cinfo);
 
 /*
  * Dissectors should never modify the packet data.
  */
 extern void dissect_file(struct epan_dissect *edt,
-    struct wtap_pkthdr *phdr, tvbuff_t *tvb,
-    frame_data *fd, column_info *cinfo);
+    wtap_rec *rec, tvbuff_t *tvb, frame_data *fd, column_info *cinfo);
 
 /* Structure passed to the ethertype dissector */
 typedef struct ethertype_data_s
 {
     guint16 etype;
-    int offset_after_ethertype;
+    int payload_offset;
     proto_tree *fh_tree;
-    int etype_id;
     int trailer_id;
     int fcs_len;
 } ethertype_data_t;
@@ -830,6 +887,20 @@ WS_DLL_PUBLIC gboolean postdissectors_want_hfids(void);
 WS_DLL_PUBLIC void
 prime_epan_dissect_with_postdissector_wanted_hfids(epan_dissect_t *edt);
 
+/** Increment the dissection depth.
+ * This should be used to limit recursion outside the tree depth checks in
+ * call_dissector and dissector_try_heuristic.
+ * @param pinfo Packet Info.
+ */
+
+WS_DLL_PUBLIC void increment_dissection_depth(packet_info *pinfo);
+
+/** Decrement the dissection depth.
+ * @param pinfo Packet Info.
+ */
+
+WS_DLL_PUBLIC void decrement_dissection_depth(packet_info *pinfo);
+
 /** @} */
 
 #ifdef __cplusplus
